/* FILE: test.c                 Copyright (c), Symmetric Research, 1998
 *
 * This program shows how to call the DSPMATH functions from C.
 *
 *
 * NOTES:
 *
 * Always include dspmath.h in source code using the DSPMATH library.  It
 * ensures the functions are called with the correct arguments.
 *
 *
 *
 */

#include <stdio.h>
#include <stdlib.h>

#include "dspmath.h"



#define PI      3.141592654




/* Vector test data. */

#define N       4

float x[N] = { 1.0, 2.0, 3.0, 4.0 };
float y[N] = { 2.0, 5.0, 1.5, 0.5 };
float z[N] = { 0.0, 0.0, 0.0, 0.0 };

float result;



/* Matrix multiplication test data. */

float Ammult[N*N] = {

        1.0,  2.0,  0.0,  1.0,
        3.0,  4.0,  8.0,  1.0,
        5.0,  1.0, -2.0,  0.0,
        3.0,  0.0,  9.0,  7.0,

        };


#define M       3

float xmmult[M*N] = {

        0.0,  1.0,  0.0,  1.0,
        2.0,  1.0,  3.0,  1.0,
        1.0,  2.0,  3.0, -4.0,

        };



/* Float to int test data. */

#define NFLOAT          4

float xfloat[NFLOAT] = { 54.3, -2.7, 1.001, -1.001 };

short int xsint[NFLOAT];




/* A transpose A test data. */

float Aata[N*M] = {

        1.0,  0.0,  1.0,
        3.0,  8.0,  3.0,
        6.0,  4.5, -3.0,
        1.5,  2.0,  4.0,

        };

float bata[N] = { 1.0, 2.0, 3.0, 4.0 };




/* Gaussian LU test data. */

float Agauss[N*N] = {

          5.00000,  1.00000,  1.00000,  1.00000,
         10.00000,  3.00000,  4.00000,  3.00000,
          5.00000,  4.00000,  9.00000,  5.00000,
         10.00000,  4.00000,  8.00000,  6.00000,

        }, Agauss_LU[N*N];

float xgauss[N] = { 0.0, 0.0, 0.0, 0.0 };
float bgauss[N] = { 1.0, 3.0, 5.0, 6.0 };

        unsigned long pivot[N] = { 0, 0, 0, 0 };




/* Tridiag test data. */

float u[N] = { 15.00000,  1.00000,  2.00000,  0.00000 };
float d[N] = {  5.00000, 13.00000,  3.00000,  4.00000 };
float l[N] = {  0.00000,  4.00000,  2.00000,  1.00000 };

float b[N] = {  5.00000,  5.00000,  3.00000,  3.00000 };




/* FFT test data for small dimension. */

#define NFFT    16

COMPLEX Afft_src[NFFT] = {

        0.0,    0.0,
        1.0,    0.0,
        0.0,    0.0,
        0.0,    0.0,
        0.0,    0.0,
        0.0,    0.0,
        0.0,    0.0,
        0.0,    0.0,
        0.0,    0.0,
        0.0,    0.0,
        0.0,    0.0,
        0.0,    0.0,
        0.0,    0.0,
        0.0,    0.0,
        0.0,    0.0,
        0.0,    0.0,

        }, Afft_dest[NFFT];



/* FFT test data for larger dimension benchmark. */

#define NBENCH  1024

COMPLEX Afft_bench_src[NBENCH] = {

        {0.0, 0.0},
        {1.0, 0.0},
                
        /* the rest is zeroed off by the compiler ... */
                
        }, Afft_bench_dest[NBENCH];




/* Spectrum test data. */

#define NSPECTRUM       16

float

    taper[NSPECTRUM] = {

            1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0,
            1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0,

            },

  xspectrum[NSPECTRUM] = {
  
            1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0,
            1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0,

            };




/* FIR test data. */

#define NFIR            5
#define NFILTER         3

float

    leader[NFILTER] = { 0.0 }, xseries[NFIR] = { 0.0, 1.0, 2.0, 3.0, 4.0 },

            h[NFILTER] = { 0.0, 1.0, 0.0 }, yseries[NFIR];





/* Polynomial test data. */
        
#define NC              3

float c[NC] = { 1.0, 2.0, 3.0 };

        double xstart = 0.0, delta = .12;

#define NPOLY           5

float ypoly[NPOLY];



/* Sine test data. */

#define NSIN            5

float ysine[NSIN];



/* Cosine test data. */

#define NCOS            5

float ycosine[NCOS];



/* Vfunc test data. */

#define NFUNC           4

float
        xfunc[NFUNC] = { 1.0, 2.0, 3.0, 4.0 }, yfunc[NFUNC];



/* Interpolation test data. */

float xintrpl[5] = { 1.0, 5.0, 15.0, 0.0, 0.0 };



/* Polar to cartesian test data. */

float rtheta[4] = { 2.0, 0.0, 3.0, 2.5 }, xy[4];





/* Helper functions for printing data out. */

void printv( char *msg, float *v, unsigned n) {

        int i;

        /* Print a single precision floating point vector. */

        printf( msg);

                for ( i = 0 ; i < n ; i++ )
                        printf( " %6.2f", v[i]);

        printf( "\n");

}
void printl( char *msg, unsigned long *v, unsigned n) {

        int i;
        
        /* Print a long integer vector. */
        
        printf( msg);
        
                for ( i = 0 ; i < n ; i++ )
                        printf( " %6ld", v[i]);
                
        printf( "\n");

}
void printsi( char *msg, short int *v, unsigned n) {

        int i;
        
        /* Print a unsigned integer vector. */
        
        printf( msg);
        
                for ( i = 0 ; i < n ; i++ )
                        printf( " %6hd", v[i]);
                
        printf( "\n");

}






/* Main program to test the library functions. */
        
#define VECTOR          "Vector functions\n"

#define ADD             "     add (should be  3.00  7.00 4.50 4.50):"
#define SUB             "     sub (should be -1.00 -3.00 1.50 3.50):"
#define XPCY            "    xpcy (should be  5.00 12.00 6.00 5.00):"
#define SCALE           "   scale (should be  2.00  4.00 6.00 8.00):"
#define INNER           "                   inner (should be 37.00):"
#define VMULT           "    vmult (should be 4.00 20.00 9.00 4.00):"

#define MMULT           "Matrix multiplication\n"
#define MMULT1          "   b (should be  3.00  5.00  1.00  7.00):"
#define MMULT2          "   b (should be  5.00 35.00  5.00 40.00):"
#define MMULT3          "   b (should be  1.00 31.00  1.00  2.00):"

#define FLOAT_TO_INT    "Float to integer conversion\n"
#define FLOAT_TO_INT1   "   x (should be  54  -3  1  -1):"

#define ATA             "A transpose times A\n"
#define ATA1            "   b (should be  31.00  37.50  14.00):"
#define ATA2            "   A (should be  48.25  54.00  -2.00):"
#define ATA3            "   A (should be  54.00  88.25  18.50):"
#define ATA4            "   A (should be  -2.00  18.50  35.00):"

#define GAUSSIAN        "Gaussian LU matrix solution WITHOUT pivoting\n"

#define GAUSS1          "  A (should be  5.00  1.00  1.00  1.00):"
#define GAUSS2          "  A (should be  2.00  1.00  2.00  1.00):"
#define GAUSS3          "  A (should be  1.00  3.00  2.00  1.00):"
#define GAUSS4          "  A (should be  2.00  2.00  1.00  1.00):"
#define GAUSSX          "  x (should be  0.00  0.00  0.00  1.00):"
#define GAUSSB          "  b (should be  1.00  3.00  5.00  6.00):"
#define GAUSSP          "  p (should be     0     0     0     0):"

#define GAUSSIAN_P      "Gaussian LU matrix solution WITH pivoting\n"

#define GAUSS1_P        "  A (should be 10.00  4.00  8.00  6.00):"
#define GAUSS2_P        "  A (should be  0.50  2.00  5.00  2.00):"
#define GAUSS3_P        "  A (should be  1.00 -0.50 -1.50 -2.00):"
#define GAUSS4_P        "  A (should be  0.50 -0.50  0.33 -0.33):"
#define GAUSSX_P        "  x (should be  0.00  0.00  0.00  1.00):"
#define GAUSSB_P        "  b (should be  1.00  3.00  5.00  6.00):"
#define GAUSSP_P        "  p (should be     3     2     2     3):"

#define TRIDIAGONAL     "Tridiagonal matrix inverson\n"

#define TRIDIAGU        " tridiag u (should be  3.00  1.00  2.00  0.00):"
#define TRIDIAGD        " tridiag d (should be  5.00  1.00  1.00  2.00):"
#define TRIDIAGL        " tridiag l (should be  0.00  4.00  2.00  1.00):"
#define TRIDIAGB        " tridiag b (should be -5.00  2.00 -1.00  1.00):"

#define FFT_FORWARD     "FFT( 0 1 0 0 ... ), forward isign = +1:\n"
#define FFT_INVERSE     "FFT( 0 1 0 0 ... ), inverse isign = -1:\n"

#define FFT1            "\tAfft[%2d] = ( %+6.4f, %+6.4f)\n"

#define FFT_BENCH       "FFT timing, computing 50 (1024) ffts, start -> "

#define SPECTRUM        "Spectrum of a real function\n"
#define SPECTRUM1       "         x (should be 32.00 0.00 0.00 0.00):"

#define FIR             "FIR filter test\n"
#define FIR1            "    y (should be 0.00 0.00 1.00 2.00 3.00):"

#define POLY            "Polynomial evaluation\n"
#define POLY_Y          "    y (should be 1.00 1.28 1.65 2.11 2.65):"

#define SIN_ARRAY       "Sine array\n"
#define SIN_ARRAY1      "   y (should be 0.00 0.87 0.87 0.00 -0.87):"

#define COS_ARRAY       "Cosine array\n"
#define COS_ARRAY1      " y (should be 1.00 0.50 -0.50 -1.00 -0.50):"

#define VSIN            "Vector sin test\n"
#define VSIN1           "      y (should be 0.84  0.91  0.14 -0.76):"

#define VCOS            "Vector cos test\n"
#define VCOS1           "      y (should be 0.54 -0.42 -0.99 -0.65):"

#define VINV            "Vector inv test\n"
#define VINV1           "      y (should be 1.00  0.50  0.33  0.25):"

#define VSQRT           "Vector sqrt test\n"
#define VSQRT1          "      y (should be 1.00  1.41  1.73  2.00):"

#define VLOG10          "Vector log10 test\n"
#define VLOG101         "      y (should be 0.00  0.30  0.48  0.60):"

#define INTERPOLATE     "Linear interpolation\n"
#define INTERPOLATE1    "  x (should be 1.00 3.00 5.00 10.00 15.00):"

#define POLAR_TO_XY     "Polar to cartesian coordinates\n"
#define POLAR_TO_XY1    "       xy (should be 2.00 0.00 -2.40 1.80):"


int main( void) {


        int i;


            float result; COMPLEX *asrc, *adest;




        /* Initialize and test the DSPMATH library. */
        
        if ( !init_dspmath() )

                { printf( "ERROR: unable to initialize DSPMATH"); exit( 1); }


        printf( "Testing the DSPMATH math library functions ...\n\n");




        /* Test the vector routines. */

        printf( VECTOR);

                  add( x, y, N, z);             printv( ADD, z, N);
                  sub( x, y, N, z);             printv( SUB, z, N);
                 xpcy( x, 2.0, y, N, z);        printv( XPCY, z, N);
                scale( x, N, 2.0, x);           printv( SCALE, x, N);
                inner( x, y, N, &result);       printv( INNER, &result, 1);
                vmult( x, y, N, z);             printv( VMULT, z, N);

                printf( "\n");




        /* Test the matrix multiplier mmult. */
                
        printf( MMULT);
        
        mmult( Ammult, xmmult, N, M, xmmult);

                printv( MMULT1, &xmmult[0*N], N);
                printv( MMULT2, &xmmult[1*N], N);
                printv( MMULT3, &xmmult[2*N], N);

                printf( "\n");




        /* Test scaling from floats to ints. */
                
        printf( FLOAT_TO_INT);
        
        iscale( xfloat, NFLOAT, 1.0, xsint, 1); /* chose round format */
        
                printsi( FLOAT_TO_INT1, xsint, NFLOAT);
                
                printf( "\n");
                



        /* Test A tranpose A. */
                
        printf( ATA);
        
        AtranA( Aata, bata, N, M, Aata, bata);

                printv( ATA1, bata, M);
                printv( ATA2, &Aata[0*M], M);
                printv( ATA3, &Aata[1*M], M);
                printv( ATA4, &Aata[2*M], M);

                printf( "\n");




        /* Test the gaussian matrix solver WITHOUT pivoting. */
                
        printf( GAUSSIAN);

        gauss( Agauss, bgauss, N, Agauss_LU, xgauss, NULL);
        
                printv( GAUSS1, &Agauss_LU[0], N);
                printv( GAUSS2, &Agauss_LU[4], N);
                printv( GAUSS3, &Agauss_LU[8], N);
                printv( GAUSS4, &Agauss_LU[12], N);

                printv( GAUSSX, xgauss, N);
                printv( GAUSSB, bgauss, N);
                
                printl( GAUSSP, pivot, N);              
                
                printf( "\n");




        /* Test the gaussian matrix solver WITH pivoting. */
                
        printf( GAUSSIAN_P);

        gauss( Agauss, bgauss, N, Agauss_LU, xgauss, pivot);

                printv( GAUSS1_P, &Agauss_LU[0], N);
                printv( GAUSS2_P, &Agauss_LU[4], N);
                printv( GAUSS3_P, &Agauss_LU[8], N);
                printv( GAUSS4_P, &Agauss_LU[12], N);

                printv( GAUSSX_P, xgauss, N);
                printv( GAUSSB_P, bgauss, N);

                printl( GAUSSP_P, pivot, N);            
                
                printf( "\n");




        /* Test the tridiagonal matrix solver. */
        
        printf( TRIDIAGONAL);

        tridiag( u, d, l, b, N, u, d, l, x);
        
                printv( TRIDIAGU, u, N);
                printv( TRIDIAGD, d, N);
                printv( TRIDIAGL, l, N);
                printv( TRIDIAGB, x, N);

                printf( "\n");




        /* Test the forward FFT with a small dimension. */

        printf( FFT_FORWARD);

        fourier( Afft_src, NFFT, Afft_dest, +1);
        
                for ( i = 0 ; i < NFFT ; i++ )
                        printf( FFT1, i, Afft_dest[i].r, Afft_dest[i].i);

                printf( "\n");





        /* Test the inverse FFT with a small dimension. */

        printf( FFT_INVERSE);

        fourier( Afft_src, NFFT, Afft_dest, -1);
        
                for ( i = 0 ; i < NFFT ; i++ )
                        printf( FFT1, i, Afft_dest[i].r, Afft_dest[i].i);

                printf( "\n");




        /* Benchmark the FFT with a larger dimension. */
        
        printf( FFT_BENCH);

        for ( i = 0 ; i < 50 ; i++ )

                fourier( Afft_bench_src, NBENCH, Afft_bench_dest, +1);

        printf( "stop\n\n");





        /* Test computing a spectrum. */
        
        printf( SPECTRUM);

        spectrum( xspectrum, taper, NSPECTRUM, xspectrum);
        
                printv( SPECTRUM1, xspectrum, 4);

                printf( "\n");




        /* Test the FIR filter. */
                
        printf( FIR);
        
        fir( leader, xseries, NFIR, h, NFILTER, yseries);
        
                printv( FIR1, yseries, NFIR);
                
                printf( "\n");
                
                


        /* Test the polynomial evaluator. */
        
        printf( POLY);
        
        poly_array( c, NC, xstart, delta, ypoly, NPOLY);
        
                printv( POLY_Y, ypoly, NPOLY);
                
                printf(" \n");




        /* Test the sine function. */
                
        printf( SIN_ARRAY);

        sin_array( 0.0, PI/3.0, ysine, NSIN);

                printv( SIN_ARRAY1, ysine, NSIN);

                printf( "\n");




        /* Test the cosine function. */

        printf( COS_ARRAY);

        cos_array( 0.0, PI/3.0, ycosine, NCOS);

                printv( COS_ARRAY1, ycosine, NCOS);

                printf( "\n");



        /* Test the vector sin function. */

        printf( VSIN);
        
        vsin( xfunc, NFUNC, yfunc);
        
                printv( VSIN1, yfunc, NFUNC);
                
                printf( "\n");




        /* Test the vector cos function. */

        printf( VCOS);
        
        vcos( xfunc, NFUNC, yfunc);
        
                printv( VCOS1, yfunc, NFUNC);
                
                printf( "\n");




        /* Test the vector inverse function. */

        printf( VINV);
        
        vinv( xfunc, NFUNC, yfunc);
        
                printv( VINV1, yfunc, NFUNC);
                
                printf( "\n");




        /* Test the vector square root function. */

        printf( VSQRT);
        
        vsqrt( xfunc, NFUNC, yfunc);
        
                printv( VSQRT1, yfunc, NFUNC);
                
                printf( "\n");




        /* Test the vector log10 function. */

        printf( VLOG10);
        
        vlog10( xfunc, NFUNC, yfunc);
        
                printv( VLOG101, yfunc, NFUNC);
                
                printf( "\n");




        /* Test the interpolation function. */
                
        printf( INTERPOLATE);
        
        interpolate( xintrpl, 3, xintrpl, 5);
        
                printv( INTERPOLATE1, xintrpl, 5);
                
                printf( "\n");




        /* Test the polar to cartesian coordinate conversion. */
                
        printf( POLAR_TO_XY);
        
        polar_to_xy( rtheta, 4, xy);
        
                printv( POLAR_TO_XY1, xy, 4);
                
                printf( "\n");


        exit( 0);
}
